/*
Weather Station - G2C Demo Project

    Date          : Oct 2019
    Author        : Nemanja Medakovic

Test configuration STM32 :

    MCU              : STM32F407ZG
    MCU Clock        : 168MHz
    Dev. Board       : Fusion for STM32 (ARM) v8
    TFT              : 5"
    ARM Compiler ver : v6.2.0.0

---

Description :

The application is composed of three sections :

- System Initialization - Initializes all necessary GPIO pins, TFT display,
  UART module to communicate with G2C click and I2C used for the communication with sensors.
  The TFT display, which is used in this project, is 5 inches size, or 800x480.
- Application Initialization - Initializes all sensors drivers, G2C timer,
  then turns ON the TFT work screen.
  All Click boards will be initialized in this function.
- Application Task - Weather results will be updated on TFT display and sent to
  ClickCloud web application every one second if they were changed.
  Weather Station can measure temperature, humidity, ambient light, IR light and UV light.
  This project also consists informations about time and date.
  Actuator enable/disable, on ClickCloud application, is used to control results
  displaying, to be showed on display and updated on web, or not.
  
Note: The USB UART, on development board, must be switched OFF.
      Also user can enable PORTC (upper bits) to see interaction between Weather Station
      and ClickCloud application.

Additional functions:

- displayCtrl - This function performs a display control by checking the A0 button state.
- initTFT - This function initializes TFT display and turns ON the start screen.
- waitingProcess - This function executes a waiting process on TFT display.
- updateLabel - This function is used to change (update) the selected label on TFT display.
- g2c_packCmd - This function sends a command packet to G2C click.

All additional functions such as timer initialization and default handler.

*/

#include "Click_G2C_types.h"
#include "Click_G2C_config.h"
#include "Click_G2C_timer.h"
#include "Click_Ambient6_types.h"
#include "Click_RTC_7_types.h"
#include "Click_RTC_7_config.h"
#include "Click_TempHum8_types.h"
#include "Click_UV_4_types.h"
#include "Click_UV_4_config.h"
#include "TFT_objects.h"
#include "TFT_resources.h"
#include "cstring.h"

/** ClickCloud actuator and sensors reference
 *
 * NOTE: By changing this parameters user can modify the actuator and sensor reference.
 */
static char g2c_actuatorRef[ 10 ] = "RTC7_EN";

static char g2c_tempRef [ 15 ] = "TEMPHUM8_T";
static char g2c_humRef  [ 15 ] = "TEMPHUM8_H";
static char g2c_lightRef[ 15 ] = "AMBIENT6_LX";
static char g2c_uvRef   [ 10 ] = "UV4_UV";
static char g2c_irRef   [ 10 ] = "UV4_SIR";
static char g2c_hourRef [ 10 ] = "RTC7_H";
static char g2c_minRef  [ 10 ] = "RTC7_MIN";
static char g2c_weekRef [ 10 ] = "RTC7_WEEK";
static char g2c_dayRef  [ 10 ] = "RTC7_DAY";
static char g2c_monthRef[ 15 ] = "RTC7_MONTH";
static char g2c_yearRef [ 10 ] = "RTC7_YEAR";

/** ClickCloud network and device info
 *
 * NOTE: By changing this parameters user can modify the network and ClickCloud device
 *       configurations.
 */
static char g2c_networkName     [ 20 ]  = "Network Name";
static char g2c_networkPassword [ 20 ]  = "Network Password";
static char g2c_devKey          [ 20 ]  = "dox8zkrv5jc3p809";
static char g2c_devPass         [ 50 ]  = "9e81b16b-c76f-47f4-afdb-9ce48eeb5b25";

/** Control variables to enable/disable data update */
static uint8_t state;
static uint8_t oldstate;

/** G2C commands */
static char _ATE    [ 5 ] = "ATE1";
static char _AT     [ 3 ] = "AT";
static char _AT_CEN [ 9 ] = "AT+CEN=1";
static char _AT_NWCR[ 8 ] = "AT+NWCR";
static char _AT_NWC [ 9 ] = "AT+NWC=1";
static char _AT_BRCR[ 8 ] = "AT+BRCR";
static char _AT_BRC [ 9 ] = "AT+BRC=1";
static char _AT_DSET[ 8 ] = "AT+DSET";
static char _AT_PUB [ 7 ] = "AT+PUB";

uint16_t tx_idx;
static uint8_t first_write;

static TImage * image;
static uint8_t * font;

/** Data where stored the previous measurement results (needed for data update comparison) */
static uint8_t prev_sec;
static uint8_t prev_min;
static uint8_t prev_hour;
static uint8_t prev_week;
static uint8_t prev_day;
static uint8_t prev_month;
static uint8_t prev_year;
static int8_t prev_temp;
static uint8_t prev_hum;
static uint16_t prev_light;
static uint16_t prev_uv;
static uint16_t prev_ir;

/** Data measurement results */
static rtc7_time_t time_set;
static rtc7_time_t time_date;
static int8_t temperature;
static uint8_t humidity;
static uint16_t ambient_light;
static uint16_t uv_data[ 6 ];

void initTFT()
{
    uint16_t i;

    TFT_Init_SSD1963_Board_50( 800, 480 );
    TFT_Set_DBC_SSD1963( 0 );
    TFT_Set_Pen( CL_ORANGE, 1 );
    TFT_Set_Font( Montserrat41x51_Regular, CL_ORANGE, FO_HORIZONTAL );
    Start_TP();
    TFT_Fill_Screen( CL_BLACK );
    TFT_H_Line( 0, 799, 0 );
    TFT_V_Line( 1, 479, 799 );
    TFT_H_Line( 798, 0, 479 );
    TFT_V_Line( 478, 1, 0 );
    TFT_Write_Text( "Initializing", 320, 230 );

    /** Display turn ON effect */
    for (i = 1; i < 256; i++)
    {
        TFT_Set_DBC_SSD1963( i );
        Delay_ms( 2 );
    }
}

void waitingProcess()
{
    uint16_t i;
    uint16_t j;
    uint16_t k;

    for (k = 0; k < 9; k++)
    {
        j = 0;
        for (i = 0; i < 800; i++)
        {
            TFT_Set_Pen( CL_BLACK, 1 );
            TFT_H_Line( j, j + 92, 1 );

            TFT_Set_Pen( CL_ORANGE, 1 );
            TFT_H_Line( i, i + 30, 1 );
            TFT_Set_Pen( CL_YELLOW, 1 );
            TFT_H_Line( i + 31, i + 61, 1 );
            TFT_Set_Pen( CL_WHITE, 1 );
            TFT_H_Line( i + 62, i + 92, 1 );

            j = i;
            Delay_ms( 1 );
        }

        j = 0;
        for (i = 0; i < 800; i++)
        {
            TFT_Set_Pen( CL_BLACK, 1 );
            TFT_H_Line( j, j + 92, 478 );

            TFT_Set_Pen( CL_ORANGE, 1 );
            TFT_H_Line( i, i + 30, 478 );
            TFT_Set_Pen( CL_YELLOW, 1 );
            TFT_H_Line( i + 31, i + 61, 478 );
            TFT_Set_Pen( CL_WHITE, 1 );
            TFT_H_Line( i + 62, i + 92, 478 );

            j = i;
            Delay_ms( 1 );
        }
    }
}

void updateLabel( uint8_t * new_label, uint16_t x_pos, uint16_t y_pos )
{
    DrawImage( image );
    TFT_Set_Font( font, CL_WHITE, FO_HORIZONTAL );
    TFT_Write_Text( new_label, x_pos, y_pos );
}

void g2c_default_handler( uint8_t *rsp, uint8_t *evArgs )
{
    /** ACTUATOR parser */
    char cmd_act[ 5 ]     = "+ACT";
    char state_true[ 5 ]  = "true";
    char state_false[ 6 ] = "false";

    uint8_t tmpStorage[ 10 ];
    uint8_t len;

    memset( tmpStorage, 0, 10 );

    if (STR_MATCHED == _strncmp( &cmd_act[0], rsp + 2, 4 ))
    {
        len = _strlen( &g2c_actuatorRef[0] );
        if (STR_MATCHED == _strncmp( &g2c_actuatorRef[0], rsp + 9, len ))
        {
           len = len + 12;
           _strcpy( rsp + len, tmpStorage );
        }
    }
    
    if (STR_MATCHED == _strncmp( &tmpStorage[0], &state_true[0], 4 ))
    {
        uint16_t i;

        state = 1;

        /** ACTUATOR sending signalization */
        for (i = 0x0100; i < 0x4000; i <<= 1)
        {
            GPIOC_ODR |= i;
            Delay_ms( 20 );
        }
        for (i = 0x0100; i < 0x4000; i <<= 1)
        {
            GPIOC_ODR ^= i;
            Delay_ms( 20 );
        }
    }
    
    if (STR_MATCHED == _strncmp( &tmpStorage[0], &state_false[0], 5 ))
    {
        uint16_t i;

        state = 0;

        /** ACTUATOR sending signalization (receiving command from ClickCloud) */
        for (i = 0x0100; i < 0x4000; i <<= 1)
        {
            GPIOC_ODR |= i;
            Delay_ms( 20 );
        }
        for (i = 0x0100; i < 0x4000; i <<= 1)
        {
            GPIOC_ODR ^= i;
            Delay_ms( 20 );
        }
    }
}

void g2c_packCmd( uint8_t *cmd, uint8_t *mid, uint8_t *_end )
{
    char cmd_param[ 10 ] = "=\"";
    char mid_param[ 10 ] = "\",\"";
    char end_param[ 10 ] = "\"";

    uint8_t tmpBuf[ 80 ];
    memset( &tmpBuf[0], 0, 80 );

    strcat( &tmpBuf[0], cmd );
    strcat( &tmpBuf[0], &cmd_param[0] );
    strcat( &tmpBuf[0], mid );
    strcat( &tmpBuf[0], &mid_param[0] );
    strcat( &tmpBuf[0], _end );
    strcat( &tmpBuf[0], &end_param[0] );

    g2c_cmdSingle( &tmpBuf[0] );
}

void systemInit()
{
    /** G2C GPIO init */
    mikrobus_gpioInit( _MIKROBUS5, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS5, _MIKROBUS_RST_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS5, _MIKROBUS_CS_PIN,  _GPIO_OUTPUT );

    /** GPIO initialization for RTC 7 click */
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_AN_PIN,  _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_RST_PIN, _GPIO_INPUT );

    /** GPIO initialization for UV 4 click */
    mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_INT_PIN, _GPIO_INPUT );

    /** GPIO initialization for LED signalization */
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_8 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_9 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_10 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_11 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_12 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_13 );
    GPIOC_ODR.B8 = 0;
    GPIOC_ODR.B9 = 0;
    GPIOC_ODR.B10 = 0;
    GPIOC_ODR.B11 = 0;
    GPIOC_ODR.B12 = 0;
    GPIOC_ODR.B13 = 0;

    /** UART initialization for G2C click */
    mikrobus_uartInit( _MIKROBUS5, &_G2C_UART_CFG[0] );
    Delay_ms( 100 );

    /** I2C initialization for UV 4, Ambient 6 and Temp-Hum 8 clicks */
    mikrobus_i2cInit( _MIKROBUS3, &_UV4_I2C_CFG[0] );
    Delay_ms( 100 );

    /** I2C initialization for RTC 7 click */
    mikrobus_i2cInit( _MIKROBUS4, &_RTC7_I2C_CFG[0] );
    Delay_ms( 100 );

    /** TFT init */
    initTFT();
}

void applicationInit()
{
    uint16_t i;

    /** Temp-Hum 8 init */
    I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );

    temphum8_i2cDriverInit( (T_TEMPHUM8_P)&_MIKROBUS1_GPIO, (T_TEMPHUM8_P)&_MIKROBUS1_I2C, 0x40 );
    Delay_ms( 300 );
    temphum8_softReset();
    Delay_ms( 200 );
    temphum8_configuration( _TEMPHUM8_CFG_RESOLUTION_H12_T14 | _TEMPHUM8_CFG_ENABLE_ON_CHIP_HEATER | _TEMPHUM8_CFG_DISABLE_OTP_RELOAD );

    /** Ambient 6 init */
    ambient6_i2cDriverInit( (T_AMBIENT6_P)&_MIKROBUS2_GPIO, (T_AMBIENT6_P)&_MIKROBUS2_I2C, _AMBIENT6_DEVICE_SLAVE_ADDRESS );
    Delay_ms( 300 );
    ambient6_configuration( _AMBIENT6_CFG_ALS_SENSITIVITY_X2 |
                            _AMBIENT6_CFG_ALS_TIME_SETTING_25MS |
                            _AMBIENT6_CFG_PERSISTENCE_NUMBER_1 |
                            _AMBIENT6_CFG_ALS_INT_DISABLE |
                            _AMBIENT6_CFG_ALS_POWER_ON );
    ambient6_writeData( _AMBIENT6_CMD_MODE_SETTINGS, _AMBIENT6_POWER_MODE_1 | _AMBIENT6_POWER_MODE_ENABLE );
    Delay_ms( 200 );
    TFT_Write_Text( ".", 505, 230 );

    /** UV 4 init */
    uv4_i2cDriverInit( (T_UV4_P)&_MIKROBUS3_GPIO, (T_UV4_P)&_MIKROBUS3_I2C, 0x55 );
    Delay_ms( 300 );
    uv4_writeReg( _UV4_COMMAND_REG, _UV4_RESET_CTR_CMD );
    Delay_ms( 200 );
    uv4_writeParamData( _UV4_CHAN_LIST_PARAM,    _UV4_CHAN0_ENABLE | _UV4_CHAN1_ENABLE );
    uv4_writeParamData( _UV4_ADCPOST_0_PARAM,    _UV4_16BIT_OUT );
    uv4_writeParamData( _UV4_ADCPOST_1_PARAM,    _UV4_16BIT_OUT );
    uv4_writeParamData( _UV4_MEASCONFIG_0_PARAM, _UV4_MEASCOUNT1_SELECT );
    uv4_writeParamData( _UV4_MEASCONFIG_1_PARAM, _UV4_MEASCOUNT1_SELECT );
    uv4_writeParamData( _UV4_ADCCONFIG_0_PARAM,  _UV4_OPT_FUNC_UV );
    uv4_writeParamData( _UV4_ADCCONFIG_1_PARAM,  _UV4_OPT_FUNC_SMALL_IR );
    uv4_writeParamData( _UV4_BURST_PARAM,        _UV4_BURST_DISABLE );
    uv4_writeParamData( _UV4_MEASRATE_H_PARAM,   0 );
    uv4_writeParamData( _UV4_MEASRATE_L_PARAM,   0 );
    uv4_writeParamData( _UV4_MEASCOUNT_1_PARAM,  0 );

    /** RTC 7 init */
    I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );

    /** If you want to reset RTC 7 click on the selected time, modify and use this time configuration */
    time_set.seconds  = 30;
    time_set.minutes  = 46;
    time_set.hours    = 12;
    time_set.weekdays = 4;
    time_set.monthday = 10;
    time_set.month    = 10;
    time_set.year     = 19;

    rtc7_i2cDriverInit( (T_RTC7_P)&_MIKROBUS4_GPIO, (T_RTC7_P)&_MIKROBUS4_I2C, 0x69 );
    Delay_ms( 500 );
    rtc7_reset();
    rtc7_disableDst();
    rtc7_init( 0, _RTC7_24HR_FORMAT );
    rtc7_setGmtTime( &time_set );
    rtc7_setOsc( _RTC7_ENABLE_OSC, _RTC7_INPUT_FREQ_32768HZ, _RTC7_OUTPUT_FREQ_32768HZ );
    rtc7_setTimer( _RTC7_TIMER_EN, _RTC7_TIMER_FREQ_16HZ );
    TFT_Write_Text( ".", 520, 230 );

    /** TIMER INIT */
    g2c_configTimer();

    /** G2C DRIVER INIT */
    g2c_uartDriverInit( (T_G2C_P)&_MIKROBUS5_GPIO, (T_G2C_P)&_MIKROBUS5_UART );
    g2c_coreInit( g2c_default_handler, 1500 );
    g2c_modulePower( true );
    TFT_Write_Text( ".", 535, 230 );
//    g2c_moduleReset();

    /** G2C MODULE INIT */
    g2c_cmdSingle( &_ATE[0] );
    g2c_cmdSingle( &_AT[0] );
    g2c_cmdSingle( &_AT_CEN[0] );
    g2c_packCmd  ( &_AT_NWCR[0], &g2c_networkName[0], &g2c_networkPassword[0] );
    g2c_cmdSingle( &_AT_NWC[0] );
    g2c_packCmd  ( &_AT_BRCR[0], &g2c_devKey[0], &g2c_devPass[0] );
    g2c_cmdSingle( &_AT_BRC[0] );
    TFT_Write_Text( ".", 550, 230 );

    waitingProcess();

    state    = 0;
    oldstate = 0;

    prev_sec   = 60;
    prev_min   = 60;
    prev_hour  = 24;
    prev_week  = 0;
    prev_day   = 0;
    prev_month = 0;
    prev_year  = 100;
    prev_temp  = 200;
    prev_hum   = 101;
    prev_light = 65535;
    prev_uv    = 65535;
    prev_ir    = 65535;

    /** Start Work Screen */
    for (i = 255; i > 0; i--)
    {
        TFT_Set_DBC_SSD1963( i );
        Delay_ms( 3 );
    }
    TFT_Set_DBC_SSD1963( 0 );

    DrawScreen( &Screen2 );

    /** Ready signalization */
    for (i = 0x0100; i < 0x4000; i <<= 1)
    {
        GPIOC_ODR |= i;
        Delay_ms( 20 );
    }
    for (i = 0x0100; i < 0x4000; i <<= 1)
    {
        GPIOC_ODR ^= i;
        Delay_ms( 20 );
    }

    /** Show Work Screen */
    for (i = 0; i < 256; i++)
    {
        TFT_Set_DBC_SSD1963( i );
        Delay_ms( 3 );
    }
}

void applicationTask()
{
    /** CORE STATE MACHINE */
    g2c_process();

    /** START MEASUREMENT */
    if ( (state == 1) && (oldstate == 0) )
    {
        uint16_t i;

        prev_sec   = 60;
        prev_min   = 60;
        prev_hour  = 24;
        prev_week  = 0;
        prev_day   = 0;
        prev_month = 0;
        prev_year  = 100;
        prev_temp  = 200;
        prev_hum   = 101;
        prev_light = 65535;
        prev_uv    = 65535;
        prev_ir    = 65535;

        oldstate    = 1;
        first_write = 1;
    }

    /** TFT AND CLOUD MEASUREMENT RESULTS UPDATE */
    if ( (state == 1) && (oldstate == 1) )
    {
        rtc7_getLocalTime( &time_date );

        if (prev_sec != time_date.seconds)
        {
            char tx_tmp[ 20 ];
            char tx_lab[ 50 ];

            /** TIME UPDATE */
            tx_idx = 0;
            prev_sec = time_date.seconds;
            ByteToStr( time_date.hours, tx_tmp );
            Ltrim( tx_tmp );
            
            if (prev_hour != time_date.hours)
            {
                prev_hour = time_date.hours;
                g2c_packCmd( &_AT_DSET[0], &g2c_hourRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );
            }
            
            if (_strlen( tx_tmp ) == 1)
            {
                tx_tmp[ 1 ] = tx_tmp[ 0 ];
                tx_tmp[ 0 ] = '0';
                tx_tmp[ 2 ] = 0;
            }
            
            _strcpy( &tx_tmp[0], &tx_lab[0] );
            _strcpy( " : ", &tx_lab[ tx_idx ] );
            
            if (prev_min != time_date.minutes)
            {
                ByteToStr( time_date.minutes, tx_tmp );
                Ltrim( tx_tmp );

                prev_min = time_date.minutes;
                g2c_packCmd( &_AT_DSET[0], &g2c_minRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );

                if (_strlen( tx_tmp ) == 1)
                {
                    tx_tmp[ 1 ] = tx_tmp[ 0 ];
                    tx_tmp[ 0 ] = '0';
                    tx_tmp[ 2 ] = 0;
                }
                
                _strcpy( &tx_tmp[0], &tx_lab[ tx_idx ] );
                
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                
                font = Montserrat_SemiBold65x78_Regular;
                image = &Image2;
                updateLabel( tx_lab, 258, 87 );
            }

            if (prev_day != time_date.monthday)
            {
                /** DATE UPDATE */
                tx_idx = 0;
                prev_day = time_date.monthday;
                ByteToStr( time_date.month, tx_tmp );
                Ltrim( tx_tmp );
                
                if (prev_month != time_date.month)
                {
                    prev_month = time_date.month;
                    g2c_packCmd( &_AT_DSET[0], &g2c_monthRef[0], &tx_tmp[0] );
                    g2c_cmdSingle( &_AT_PUB[0] );
                }
                
                _strcpy( &tx_tmp[0], &tx_lab[0] );
                _strcpy( " / ", &tx_lab[ tx_idx ] );
                
                ByteToStr( time_date.monthday, tx_tmp );
                Ltrim( tx_tmp );
                
                g2c_packCmd( &_AT_DSET[0], &g2c_dayRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );
                
                _strcpy( &tx_tmp[0], &tx_lab[ tx_idx ] );
                _strcpy( " / ", &tx_lab[ tx_idx ] );
                
                ByteToStr( time_date.year, tx_tmp );
                Ltrim( tx_tmp );
                
                if (prev_year != time_date.year)
                {
                    prev_year = time_date.year;
                    g2c_packCmd( &_AT_DSET[0], &g2c_yearRef[0], &tx_tmp[0] );
                    g2c_cmdSingle( &_AT_PUB[0] );
                }
                
                _strcpy( &tx_tmp[0], &tx_lab[ tx_idx ] );
                
                if (prev_week != time_date.weekdays)
                {
                    prev_week = time_date.weekdays;
                    ByteToStr( time_date.weekdays, tx_tmp );
                    Ltrim( tx_tmp );
                    
                    g2c_packCmd( &_AT_DSET[0], &g2c_weekRef[0], &tx_tmp[0] );
                    g2c_cmdSingle( &_AT_PUB[0] );
                }
                
                switch (time_date.weekdays)
                {
                    case 1 :
                    {
                        _strcpy( "    MON", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 2 :
                    {
                        _strcpy( "    TUE", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 3 :
                    {
                        _strcpy( "    WED", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 4 :
                    {
                        _strcpy( "    THU", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 5 :
                    {
                        _strcpy( "    FRI", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 6 :
                    {
                        _strcpy( "    SAT", &tx_lab[ tx_idx ] );
                    break;
                    }
                    case 7 :
                    {
                        _strcpy( "    SUN", &tx_lab[ tx_idx ] );
                    }
                }
                
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                
                font = Montserrat41x51_Regular;
                image = &Image3;
                updateLabel( tx_lab, 292, 173 );
            }

            I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
            font = Montserrat41x51_Regular;

            humidity = temphum8_getHumidity();

            if (prev_hum != humidity)
            {
                /** HUMIDITY UPDATE */
                prev_hum = humidity;
                ByteToStr( humidity, tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                
                image = &Image4;
                updateLabel( tx_tmp, 136, 342 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
                
                g2c_packCmd( &_AT_DSET[0], &g2c_humRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );
            }

            temperature = temphum8_getTemperature( _TEMPHUM8_TEMPERATURE_IN_CELSIUS );

            if (prev_temp != temperature)
            {
                /** TEMPERATURE UPDATE */
                prev_temp = temperature;
                ShortToStr( temperature, tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                
                image = &Image5;
                updateLabel( tx_tmp, 383, 342 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
                
                g2c_packCmd( &_AT_DSET[0], &g2c_tempRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );
            }

            uv4_readMeasurements( &uv_data[0] );

            if (prev_ir != uv_data[1])
            {
                /** IR LIGHT UPDATE */
                prev_ir = uv_data[1];
                WordToStr( uv_data[1], tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                
                image = &Image6;
                updateLabel( tx_tmp, 660, 344 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
                
                g2c_packCmd( &_AT_DSET[0], &g2c_irRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );
            }

            if (prev_uv != uv_data[0])
            {
                /** UV LIGHT UPDATE */
                prev_uv = uv_data[0];
                WordToStr( uv_data[0], tx_tmp );
                Ltrim( tx_tmp );
                I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );
                
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                }
                
                image = &Image7;
                updateLabel( tx_tmp, 660, 204 );
                I2C_Set_Active( &I2C1_Start, &I2C1_Read, &I2C1_Write );
                
                g2c_packCmd( &_AT_DSET[0], &g2c_uvRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );
            }

            ambient_light = ambient6_getAmbientLight();
            I2C_Set_Active( &I2C2_Start, &I2C2_Read, &I2C2_Write );

            if (prev_light != ambient_light)
            {
                /** AMBIENT LIGHT UPDATE */
                prev_light = ambient_light;
                WordToStr( ambient_light, tx_tmp );
                Ltrim( tx_tmp );
                
                if (first_write == 1)
                {
                    Delay_ms( 50 );
                    first_write = 0;
                }
                
                image = &Image8;
                updateLabel( tx_tmp, 660, 79 );
                
                g2c_packCmd( &_AT_DSET[0], &g2c_lightRef[0], &tx_tmp[0] );
                g2c_cmdSingle( &_AT_PUB[0] );
            }
        }
    }

    /** STOP MEASUREMENT */
    if ( (state == 0) && (oldstate == 1) )
    {
        uint16_t i;

        for (i = 254; i > 0; i--)
        {
            TFT_Set_DBC_SSD1963( i );
            Delay_ms( 2 );
        }
        TFT_Set_DBC_SSD1963( 0 );

        DrawScreen( &Screen2 );

        for (i = 1; i < 256; i++)
        {
            TFT_Set_DBC_SSD1963( i );
            Delay_ms( 2 );
        }

        oldstate = 0;
    }
}

void main()
{
    systemInit();
    applicationInit();

    while (1)
    {
        applicationTask();
    }
}

/* -------------------------------------------------------------------------- */